/**
 * File: maze-generator.cpp
 * ------------------
 * Name:
 * Section Leader Name:
 */
#include "foreach.h"
#include <iostream>
#include "set.h"
#include "vector.h"
#include "map.h"
#include "genlib.h"
#include "extgraph.h"
#include "simpio.h"
#include "random.h"
#include "maze-types.h"
#include "maze-graphics.h"

// read a integer value between two values
int readInteger(int startI, int endI);
void buildChambers(Vector<Vector<cell> > &theChambers, int dim);
void createWall(Vector<wall> &theWalls, int dimension);
bool findCellInSubChambers(Vector<cell> &subChambers, cell findThisCell, int index =0);
int findCellInChambers(Vector<Vector<cell> > &chambers, cell findThisCell, int index=0);
void setupRandomizedWalls(Vector<wall> &theWalls);
void removeStartAndEnd(int dimension);
Vector<cell> buildChambersX(int dim, int y, int index=0);

int main() 
{
	int dimension;
	Vector<Vector<cell> > chambers;
	Vector<wall> theWalls;

	InitMazeGraphics();

	while (true)
	{
		dimension = readInteger(7,50);
		if (dimension == 0) return 0;

		InitMazeGraphics();

		buildChambers(chambers, dimension);
		createWall(theWalls, dimension);

		setupRandomizedWalls(theWalls);

		DrawMazeBorder();
		SetMazeDimension(dimension);

		// draw all of the walls, iterator though the vector list
		foreach (wall theWall in theWalls)
		{
			DrawWall(theWall);
		}

		int wallI = 0;
		// could also do a list of already checked cells since there would be potentially 4 links in
		Vector<wall>::Iterator wallIter = theWalls.iterator();
		while (wallIter.hasNext() && (wallI < ((dimension * dimension)-1)))
		{
			wall checkThisWall = wallIter.next();
			
			int firstFound = findCellInChambers(chambers, checkThisWall.one);
			if (firstFound >= 0)
			{
				if (!findCellInSubChambers(chambers[firstFound], checkThisWall.two))
				{
					wallI++;
					RemoveWall(checkThisWall);
					int found = findCellInChambers(chambers, checkThisWall.two);
					if (found >= 0)
					{
						for (int foundI = 0; foundI < chambers[found].size(); foundI++)
							chambers[firstFound].add(chambers[found][foundI]);
						chambers.removeAt(found);
					}
				}
			}
		}
		removeStartAndEnd(dimension);
	}
	return 0;
}

int readInteger(int startI, int endI)
{
	int retInt;
	while (true)
	{
		cout << "Please enter a integer value (" << startI  << " - " << endI << ") 0 to exit : ";
		retInt = GetInteger();
		if (retInt >= startI && retInt <= endI)
			return retInt;
		if (retInt ==0)
			return 0;
	}
}

// randomize the walls to pick different walls in a queue effecitively 
void setupRandomizedWalls(Vector<wall> &theWalls)
{
	Randomize();
	int rand1, rand2;
	// more the walls around, swap elements, at a random place
	for (int i = 0; i < theWalls.size(); i++)
	{
		rand1 = RandomInteger(0,(theWalls.size()-1));
		rand2 = RandomInteger(0,(theWalls.size()-1));
		// swap
		wall tmp = theWalls[rand1];
		theWalls[rand1] = theWalls[rand2];
		theWalls[rand2] = tmp;
	}
}

//build up a list of chambers with the default singlar values of each chamber
void buildChambers(Vector<Vector<cell> > &theChambers, int dim)
{
	cell newCell;
	for (int x= 0; x < dim; x++)
		for (int y = 0; y < dim; y++)
		{
			newCell.col = x;
			newCell.row = y;
			Vector<cell> newV;
			newV.add(newCell);
			theChambers.add(newV);
//			theChambers.add(buildChambersX(dim, y));
		}
}

// could do building up in recusive of cells, but I think that doing it via a loop is better.
Vector<cell> buildChambersX(int dim, int y, int index)
{
	Vector<cell> returnCell;
	cell newCell;
	if (index == (dim-1))
	{
		newCell.col =index;
		newCell.row = y;
		returnCell.add(newCell);
		return returnCell;
	}
	else
	{
		newCell.col = index;
		newCell.row = y;
		returnCell = buildChambersX(dim, y, ++index);
		returnCell.add(newCell);
		return returnCell;
	}
}

// create the walls within the grid structure
void createWall(Vector<wall> &theWalls, int dimension)
{
	wall right, down;
	// create the walls
	for (int x=0; x < dimension; x++)
		for (int y = 0; y < dimension; y++)
		{
			// going right
			if (x < (dimension-1))
			{
				right.one.col = x;
				right.one.row = y;
				right.two.col = x+1;
				right.two.row = y;
				theWalls.add(right);
			}
			// going down, if not at the bottom
			if (y < (dimension-1))
			{
				down.one.col = x;
				down.one.row = y;
				down.two.col = x;
				down.two.row = y+1;
				theWalls.add(down);
			}
		}

}

// return the main vector where the cell has been found in
int findCellInChambers(Vector<Vector<cell> > &chambers, cell findThisCell, int index)
{
	if (index < chambers.size())
	{
		if (findCellInSubChambers(chambers[index], findThisCell))
			return index;
		else
			return findCellInChambers(chambers, findThisCell, ++index);
	}
	return -1;
}

// recusive method
// return true if the cell is within the vector of cells
bool findCellInSubChambers(Vector<cell> &subChambers, cell findThisCell, int index)
{
	if (index < subChambers.size())
	{
		if (subChambers[index] == findThisCell)
			return true;
		else 
			return findCellInSubChambers(subChambers, findThisCell, ++index);
	}
	return false;
}

void removeStartAndEnd(int dimension)
{
	// remove the start and end of the maze
	wall startWall;
	startWall.one.col = 0;
	startWall.one.row = (dimension-1);
	startWall.two.col = 0;
	startWall.two.row = dimension;
	RemoveWall(startWall);
	wall endWall;
	endWall.one.col = (dimension-1);
	endWall.one.row = -1;
	endWall.two.col = (dimension-1);
	endWall.two.row = 0;
	RemoveWall(endWall);
}

//non recusive method
// return the main vector where the cell has been found in
/*int findCellInChambers(Vector<Vector<cell> > &chambers, cell findThisCell, int index)
{
	for (int mainSubChamber =0; mainSubChamber < chambers.size(); mainSubChamber++)
	{
		if (findCellInSubChambers(chambers[mainSubChamber], findThisCell))
			return mainSubChamber;
	}
	return -1;
}
*/
// non-recusive method
// return true if the cell is within the vector of cells
/*bool findCellInSubChambers(Vector<cell> &subChambers, cell findThisCell, int index)
{
	for (int mainSubSubChamber = 0; mainSubSubChamber < subChambers.size(); mainSubSubChamber++)
	{
		if (subChambers[mainSubSubChamber] == findThisCell)
		{
			return true;
		}
	}
	return false;
}
*/
